<?php

namespace Id4me\RP;

use Id4me\RP\Exception\InvalidAuthorityIssuerException;
use Id4me\RP\Exception\InvalidIDTokenException;
use Id4me\RP\Helper\Base64UrlHelper;
use Id4me\RP\Model\IdToken;
use Id4me\RP\Model\OpenIdConfig;

/**
 * Class responsible of handling entire Authorization Process toward OpenId Authority
 */
class Authorization
{
    /**
     * @var HttpClient;
     */
    private $httpClient;

    /**
     * @var \Id4me\RP\Validation;
     */
    private $validation;

    /**
     * Authorization constructor.
     *
     * @param HttpClient $client
     */
    public function __construct(HttpClient $client)
    {
        $this->setHttpClient($client);
        $this->validation = new Validation();
    }

    /**
     * Creates an authorization url out of given parameters
     *
     * @param string $authorizationEndpoint authority authorization endpoint to fetch url from
     * @param string $clientId              clientId registered by authority
     * @param string $identifier            identifier of consumer request open id data (might be a domain or user name)
     * @param string $redirectUrl           url of relying part requesting to be redirected to after successful user authentication
     * @param string $state                 parameter containing data to be used by stateful mechanisms within relying part after http redirection
     *
     * @return string
     */
    public function getAuthorizationUrl(
        string $authorizationEndpoint,
        string $clientId,
        string $identifier,
        string $redirectUrl,
        string $state = null,
        string $prompt = null
    ) {
        $query = [
            'client_id'     => $clientId,
            'login_hint'    => $identifier,
            'redirect_uri'  => $redirectUrl,
            'scope'         => 'openid',
            'response_type' => 'code'
        ];

        if (! empty($state)) {
            $query['state'] = $state;
        }

        if (! empty($prompt)) {
            $query['prompt'] = $prompt;
        }

        return sprintf(
            '%s?%s',
            $authorizationEndpoint,
            http_build_query($query, null, '&', PHP_QUERY_RFC3986)
        );
    }

    /**
     * Retrieve access from authority after access tokens' validation
     *
     * @param OpenIdConfig $openIdConfig    OpenId Config Instance containing data fetched from authority
     * @param string       $code            value of one time token generated by authority
     * @param string       $redirectUrl     url of relying part requesting to be redirected to after successful user authentication
     * @param string       $clientId        clientId registered by authority
     * @param string       $clientSecret    clientSecret registered by authority
     * @param array        $requestHeaders  Headers that need to be set for the request in the http client
     *
     * @return array
     *
     * @throws InvalidAuthorityIssuerException if iss and issuer values are not equal
     * @throws InvalidIDTokenException if provided id token value does not pass any of 13 validation steps
     *         described in ID Token Validation specifications of ID4Me Documentation
     */
    public function authorize(
        OpenIdConfig $openIdConfig,
        string $code,
        string $redirectUrl,
        string $clientId,
        string $clientSecret
    ): array {

        // Get authority validation key(s)
        $jwksArray = $this->getJwksArray($openIdConfig->getJwksUri());

        // Get authority response with authorization code
        $accessTokens = $this->getAccessTokens(
            $openIdConfig,
            $code,
            $redirectUrl,
            $clientId,
            $clientSecret
        );

        // Extract and validate ID token data
        if (! isset($accessTokens['id_token'])) {
            throw new InvalidIDTokenException('ID Token not found');
        }

        $idToken = new IdToken($accessTokens['id_token']);

        $this->validation->validateIdTokenSignature(
            $idToken->getHeaderValue('kid'),
            $idToken->getHeaderValue('alg'),
            $idToken,
            $jwksArray
        );

        $this->validation->validateISS(
            $idToken->getBodyValue('iss'),
            $openIdConfig->getIssuer()
        );

        $this->validation->validateAudience(
            $idToken->getBodyValue('aud'),
            $idToken->getBodyValue('azp'),
            $clientId
        );

        $this->validation->validateExpirationTime(
            $idToken->getBodyValue('exp')
        );

        return [
            'identifier' => $idToken->getBodyValue('id4me.identifier'),
        ];
    }

    /**
     * Retrieve JWS keys from authority
     *
     * @param string $jwksUri
     *
     * @return array
     *
     * @throws InvalidIDTokenException
     */
    public function getJwksArray(string $jwksUri): array
    {
        $response = $this->httpClient->get($jwksUri);

        if (! $response) {
            throw new InvalidIDTokenException('Unable to retrieve authority JWS keys');
        }

        return json_decode($response, true);
    }

    /**
     * Retrieve access tokens from authority
     *
     * @param OpenIdConfig $openIdConfig
     * @param string       $code
     * @param string       $redirectUrl
     * @param string       $clientId
     * @param string       $clientSecret
     * @param array        $requestHeaders  Headers that need to be set for the request in the http client
     *
     * @return array
     *
     * @throws InvalidIDTokenException
     */
    public function getAccessTokens(
        OpenIdConfig $openIdConfig,
        string $code,
        string $redirectUrl,
        string $clientId,
        string $clientSecret,
        array $requestHeaders = []
    ): array {

        $headers = array_merge(
            $requestHeaders,
            [
                'Content-Type' => 'application/x-www-form-urlencoded',
                'Authorization' => 'Basic ' . base64_encode($clientId . ':' . $clientSecret)
            ]
        );

        $response = $this->httpClient->post(
            $openIdConfig->getTokenEndpoint(),
            http_build_query([
                'code'         => $code,
                'grant_type'   => 'authorization_code',
                'redirect_uri' => $redirectUrl
            ]),
            $headers
        );

        if (! $response) {
            throw new InvalidIDTokenException('Unable to retrieve access tokens');
        }

        return json_decode($response, true);
    }

    /**
     * Set current http client to another value
     *
     * @param HttpClient $httpClient
     */
    public function setHttpClient(HttpClient $httpClient)
    {
        $this->httpClient = $httpClient;
    }

    /**
     * Set current validation handler to another value
     *
     * @param Validation $validation
     */
    public function setValidation(Validation $validation)
    {
        $this->validation = $validation;
    }
}
